﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Globalization;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;



namespace FontRendering
{
    public class CharRecord
    {
        public int ID;

        public int X;
        public int Y;
        public int Width;
        public int Height;

        public float xoffset;
        public float yoffset;
        public float xadvance;

        public CharRecord(int id, int x, int y, int w, int h, float xo, float yo, float xa)
        {
            ID = id;
            X = x;
            Y = y;
            Width = w;
            Height = h;
            xoffset = xo;
            yoffset = yo;
            xadvance = xa;
        }
    };

    public class SDFFont
    {
        float Scale;
        Texture2D texture;
        public Effect effect;
        Dictionary<int, CharRecord> chars = new Dictionary<int, CharRecord>();
        GlyphBatch gbatch;

        public SDFFont(GraphicsDevice device, String description, Texture2D tex, Effect eff, float scale = 0.4f)
        {
            char[] trims = new char[] { ' ', '\t' };
            char[] splits = new char[] { ' ', '=', '\t' };
            texture = tex;
            effect = eff;
            using (TextReader reader = File.OpenText(description))
            {
                String line = reader.ReadLine();
                line = reader.ReadLine();
                while ((line = reader.ReadLine()) != null)
                {
                    string[] parts = line.Split(splits, StringSplitOptions.RemoveEmptyEntries);
                    int i = int.Parse(parts[2].TrimEnd(trims));
                    int x = int.Parse(parts[4].TrimEnd(trims));
                    int y = int.Parse(parts[6].TrimEnd(trims));
                    int w = int.Parse(parts[8].TrimEnd(trims));
                    int h = int.Parse(parts[10].TrimEnd(trims));
                    float xo = float.Parse(parts[12].TrimEnd(trims), System.Globalization.CultureInfo.InvariantCulture);
                    float yo = float.Parse(parts[14].TrimEnd(trims), System.Globalization.CultureInfo.InvariantCulture);
                    float xa = float.Parse(parts[16].TrimEnd(trims), System.Globalization.CultureInfo.InvariantCulture);
                    chars.Add(i, new CharRecord(i, x, y, w, h, xo, yo, xa));
                }
            }
            Scale = scale;

            gbatch = new GlyphBatch(device);
        }

        public void DrawString(String s, Vector2 location)
        {
            foreach (Char c in s)
            {
                CharRecord cr = chars[(int)c];
                Vector4 src = new Vector4((float)cr.X / (float)texture.Width, (float)cr.Y / (float)texture.Height, (float)cr.Width / (float)texture.Width, (float)cr.Height / texture.Height);
                Vector4 dst = new Vector4((location.X + (cr.xoffset * Scale)), (location.Y - (cr.yoffset * Scale)), ((cr.Width * Scale)), ((cr.Height * Scale)));

                gbatch.Draw(src, dst);
                location.X += (cr.xadvance) * Scale;
            }
        }
        public void StartFontRendering(Color color)
        {
            gbatch.Start(texture, effect, color);
        }

        public void EndFontRendering()
        {
            gbatch.End();
        }

        public Vector2 MeasureString(String s)
        {
            Vector2 result = new Vector2(0, 0);
            foreach (Char c in s)
            {
                CharRecord cr = chars[(int)c];
                float ch = cr.Height;
                if (ch > result.Y)
                    result.Y = ch;
                result.X += cr.xadvance;
            }

            return result * Scale;
        }
    }
}
